package s3

import (
	"context"
	"github.com/aws/aws-sdk-go-v2/aws"
	"github.com/aws/aws-sdk-go-v2/service/s3"
	"github.com/aws/aws-sdk-go-v2/service/s3/types"
	"github.com/samber/lo"
	"strings"
	"time"
)

type ListBucketRes struct {
	CreatedAt int64  `json:"created_at"`
	Name      string `json:"name"`
}

type ListFileRes struct {
	Name         string
	LastModified time.Time
	Size         int64
}

func (c *Client) ListBucket(ctx context.Context) ([]*ListBucketRes, error) {
	var (
		err   error
		input = &s3.ListBucketsInput{
			MaxBuckets: aws.Int32(100),
		}
		output *s3.ListBucketsOutput
	)

	if output, err = c.client.ListBuckets(ctx, input); err != nil {
		return nil, err
	}

	res := lo.Map(
		output.Buckets,
		func(item types.Bucket, index int) *ListBucketRes {
			return &ListBucketRes{CreatedAt: item.CreationDate.UnixMilli(), Name: *item.Name}
		},
	)

	return res, nil
}

func (c *Client) ListFile(ctx context.Context, bucket string, prefix string, parent string) ([]*ListFileRes, error) {
	var (
		err   error
		input = &s3.ListObjectsV2Input{
			Delimiter: aws.String("/"),
			MaxKeys:   aws.Int32(1000),
			Bucket:    aws.String(bucket),
		}
		output *s3.ListObjectsV2Output
	)

	if prefix != "" {
		input.Prefix = aws.String(prefix)
	}

	if output, err = c.client.ListObjectsV2(ctx, input); err != nil {
		return nil, err
	}

	folder := lo.FilterMap(
		output.CommonPrefixes,
		func(item types.CommonPrefix, index int) (*ListFileRes, bool) {
			name := strings.TrimPrefix(*item.Prefix, parent)
			return &ListFileRes{
				Name: name,
			}, name != ""
		},
	)

	list := lo.Map(
		output.Contents,
		func(item types.Object, index int) *ListFileRes {
			return &ListFileRes{
				Name:         *item.Key,
				LastModified: *item.LastModified,
				Size:         *item.Size,
			}
		},
	)

	return append(folder, list...), nil
}
